/*
 * @FilePath: /modelTRT_custom_plugin/modelConverter/inc/ConverterInterface.hpp
 * @Brief:
 * @Version: 1.0
 * @Date: 2021-03-05 05:49:17
 * @Author: WangJiaHao
 * @Copyright: IChinae(BEIJING) TECH.CO.,LTD.
 * @LastEditors: Wangjiahao
 * @LastEditTime: 2021-03-10 01:43:50
 */
#ifndef CONVERTER_INTERFACE_H_
#define CONVERTER_INTERFACE_H_
#include<NvInfer.h>
#include"nlohmann/fifo_map.hpp"
#include<string>
#include<fstream>
#include<memory>
#include<glog/logging.h>

struct ConvertParamsInterface
{
    bool isInt8Mode = false;
    std::string weightspath{};
    std::string out_folder;
    std::string name{""};
    int max_work_space_mbi{1024};
    int max_batch_size{1};
    nlohmann::fifo_map<std::string, nvinfer1::Dims> inputlayers;
    nlohmann::fifo_map<std::string, nvinfer1::Dims> outputLayers;
    bool autoDynamicShape = false;
    virtual void fake() {};

};
class ConverterInterface
{
public:
    virtual bool convert() = 0;
    bool dumpEngine(const std::string& path)
    {
        if (engine_ != nullptr)
        {
            auto mem = engine_->serialize();
            if (mem->data() == nullptr)
            {
                LOG(ERROR) << "[ConverterInterface]---->cannot serialize model:" << paramsbase_->name;
                return false;
            }
            else
            {
                std::ofstream engineFile(path, std::ios::binary);
                if (!engineFile)
                {
                    LOG(ERROR) << "[ConverterInterface]---->cannot create file:" << path;
                    mem->destroy();
                    return false;
                }
                else
                {
                    engineFile.write(static_cast<char*>(mem->data()), mem->size());
                    if (!engineFile.fail())
                    {
                        engineFile.close();
                        mem->destroy();
                        return true;
                    }
                    else
                    {
                        LOG(ERROR) << "[ConverterInterface]----> file:" << path << " write error!";
                        engineFile.close();
                        mem->destroy();
                        return false;
                    }
                }
            }
        }
        else
        {
            LOG(ERROR) << "[ConverterInterface]---->model not convert!";
            return false;
        }
    }
    virtual bool setConvertParams(const std::shared_ptr<ConvertParamsInterface>& params)
    {
        paramsbase_ = params;
        return true;
    };
protected:

    std::shared_ptr<nvinfer1::ICudaEngine> engine_{nullptr};
    std::shared_ptr<ConvertParamsInterface> paramsbase_;
};
#endif