/*
 * @FilePath: /modelTRT_custom_plugin/modelConverter/src/caffeConverter.cpp
 * @Brief:
 * @Version: 1.0
 * @Date: 2021-03-05 08:03:39
 * @Author: WangJiaHao
 * @Copyright: IChinae(BEIJING) TECH.CO.,LTD.
 * @LastEditors: Wangjiahao
 * @LastEditTime: 2021-03-11 07:59:26
 */
#include"caffeConverter.hpp"
#include"dataStruct.hpp"
#include<exception>
#include"common.hpp"
bool CaffeConverter::setConvertParams(const std::shared_ptr<ConvertParamsInterface>& params)
{
    try
    {
        //check type valid
        dynamic_cast<CaffeConverterParams*>(params.get());
        paramsbase_ = params;
        return true;
    }
    catch (const std::exception& err)
    {
        LOG(ERROR) << "[CaffeConverter]---->setConvertParams: error type of params";
        return false;
    }
}
bool  CaffeConverter::convert()
{
    auto caffeParams = dynamic_cast<CaffeConverterParams*>(paramsbase_.get());
    if (caffeParams->name.size() == 0)
    {
        LOG(ERROR) << "[CaffeConverter]---->convert valid param not setted";
        return false;
    }
    //todo: add code for log
    LOG(INFO) << "[caffeConvert] converter init";
    builder.reset( nvinfer1::createInferBuilder(glogger));
    network.reset(builder->createNetworkV2(1U << static_cast<uint32_t>
                                           (nvinfer1::NetworkDefinitionCreationFlag::kEXPLICIT_BATCH)));
    
    config.reset(builder->createBuilderConfig());
    parser.reset(nvcaffeparser1::createCaffeParser());
    engine_ = nullptr;
    parser->setPluginFactoryV2(pluginFactory.get());
    LOG(INFO) << "[caffeConvert] converter init finished";
    LOG(INFO) << "[caffeConvert] parse model";
    const nvcaffeparser1::IBlobNameToTensor* blobNameToTensor = parser->parse(caffeParams->netWorkPath.c_str(),
            caffeParams->weightspath.c_str(), *network, nvinfer1::DataType::kFLOAT);
    int nbinputs = network->getNbInputs();
    int nboutputs = network->getNbOutputs();
    builder->setMaxBatchSize(caffeParams->max_batch_size);
    for (int i = 0; i != nbinputs; i++)
    {
        auto inputTensor = network->getInput(i);
        LOG(INFO) << "[CaffeConverter]-model:" << caffeParams->name << " input:" << i << " name:" << inputTensor->getName() <<
                  " shape:" <<
                  print_dim(inputTensor->getDimensions());

    }
    LOG(INFO) << "[caffeConvert] resetting outputs";
    std::vector<nvinfer1::ITensor*>modelOutPutTensors;
    std::vector<nvinfer1::ITensor*>targetModelOutPutTensors;
    for (auto& pair : caffeParams->outputLayers)
    {
        auto tensor = blobNameToTensor->find(pair.first.c_str());
        if (tensor == nullptr)
        {
            LOG(FATAL) << "[CaffeConverter]-model:" << caffeParams->name << " cannot find tensor named:" << pair.first.c_str();
        }
        else
        {
            targetModelOutPutTensors.push_back(tensor);
        }

    }
    for (int i = 0; i != nboutputs; i++)
    {
        auto tensor = network->getOutput(i);
        modelOutPutTensors.push_back(tensor);
    }
    for (auto& tensor : modelOutPutTensors)
    {
        network ->unmarkOutput(*tensor);
    }
    for (auto& tensor : targetModelOutPutTensors)
    {
        network ->markOutput(*tensor);
        LOG(INFO) << "[CaffeConverter]-model:" << caffeParams->name << " output name:" << tensor->getName() <<
                  " shape:" <<
                  print_dim(tensor->getDimensions());
    }
    config->setMaxWorkspaceSize(static_cast<size_t>(caffeParams->max_work_space_mbi << 20));
    //todo: add code for convert andquantize
    LOG(INFO) << "[caffeConvert] build cudaEngine";
    engine_ = std::shared_ptr<nvinfer1::ICudaEngine>(builder->buildEngineWithConfig(*network, *config), NvDeleter());
    if (engine_!=nullptr){
        
        return true;
    }
    else{
        LOG(ERROR)<<"engine object null!";
        return false;
        }

}