/*
 * @FilePath: /modelTRT_custom_plugin/plugins/inc/shuffleChannel.hpp
 * @Brief:
 * @Version: 1.0
 * @Date: 2021-03-02 03:24:16
 * @Author: WangJiaHao
 * @Copyright: IChinae(BEIJING) TECH.CO.,LTD.
 * @LastEditors: Wangjiahao
 * @LastEditTime: 2021-03-11 03:37:33
 */
#ifndef SHUFFLE_CHANNEL_H_
#define SHUFFLE_CHANNEL_H_
#include<NvInfer.h>
#include"common.hpp"
#include "NvInferRuntimeCommon.h"
#include<NvInferPlugin.h>
#include<NvCaffeParser.h>
#include <string>
#include <vector>
namespace nvinfer1
{

class ShuffleChannelPlugin: public IPluginV2IOExt
{
public:
    explicit ShuffleChannelPlugin(const std::string& name, int group = 2);
    explicit ShuffleChannelPlugin(const std::string& name, const void* data, size_t length);
    ShuffleChannelPlugin() = delete;
    int getNbOutputs() const final;
    Dims getOutputDimensions(int index, const Dims* inputs, int nbInputDims) final;
    int initialize() final;
    void terminate() final;
    size_t getWorkspaceSize(int) const final
    {
        return 0;
    };
    int enqueue(int batchSize, const void* const* inputs, void** outputs, void* workspace, cudaStream_t stream) final;
    size_t getSerializationSize() const final;

    void serialize(void* buffer) const final;

    void configurePlugin(const PluginTensorDesc* in, int nbInput, const PluginTensorDesc* out, int nbOutput)  final;

    bool supportsFormatCombination(int pos, const PluginTensorDesc* inOut, int nbInputs, int nbOutputs)  const final;

    nvinfer1::DataType getOutputDataType(int index, const nvinfer1::DataType* input_types, int num_inputs) const final;

    const char* getPluginType() const final;

    const char* getPluginVersion() const final;

    void destroy() final;

    nvinfer1::IPluginV2IOExt* clone() const final;

    void setPluginNamespace(const char* pluginNamespace) final;

    const char* getPluginNamespace() const final;
    bool isOutputBroadcastAcrossBatch(int outputIndex, const bool* inputIsBroadcasted, int nbInputs) const final{
            return false;
     }
      bool canBroadcastInputAcrossBatch(int /*inputIndex*/) const final{
          return true;
      }

private:
    const std::string layerName_;
    int group_;
    Dims shape_;
    DataType dtype_;
    std::string namespace_;
};


class ShuffleChannelPluginCreator: public IPluginCreator
{
public:
    ShuffleChannelPluginCreator();

    const char* getPluginName() const override;

    const char* getPluginVersion() const override;

    const PluginFieldCollection* getFieldNames() override;

    IPluginV2IOExt* createPlugin(const char* name, const PluginFieldCollection* fc) override;

    IPluginV2IOExt* deserializePlugin(const char* name, const void* serialData, size_t serialLength) override;

    void setPluginNamespace(const char* pluginNamespace) override;

    const char* getPluginNamespace() const override;

private:
    static PluginFieldCollection fc_;
    static std::vector<PluginField> pluginAttributes_;
    std::string namespace_;
};



REGISTER_TENSORRT_PLUGIN(ShuffleChannelPluginCreator);
}
#endif