/*
 * @FilePath: /modelTRT_custom_plugin/plugins/src/pluginFactory.cpp
 * @Brief:
 * @Version: 1.0
 * @Date: 2021-03-08 07:55:44
 * @Author: WangJiaHao
 * @Copyright: IChinae(BEIJING) TECH.CO.,LTD.
 * @LastEditors: Wangjiahao
 * @LastEditTime: 2021-03-15 07:07:55
 */
#include"pluginFactory.hpp"
#include "shuffleChannel.hpp"
#include"sliceLayer.hpp"
#include<glog/logging.h>
#include<algorithm>

namespace nvinfer1
{
bool ZDXFPluginFactory:: isPluginV2(const char* name)
{
    DLOG(INFO) << "ZDXFPluginFactory [test]---->layername:" << name;
    std::string layername{name};
    return (isShuffleChannel(layername))|| (isSliceLayer(layername));
}
//get the number from the str "XXX64"
int ZDXFPluginFactory::getLayerIndex(const std::string&layername,const std::string & prefix){
    std::string namelower(layername);
    std::string prefixlower(prefix);
    std::transform(namelower.begin(), namelower.end(), namelower.begin(), ::tolower);
    std::transform(prefixlower.begin(), prefixlower.end(), prefixlower.begin(), ::tolower);
    size_t prefix_idx=namelower.find(prefixlower);
    if(prefix_idx==std::string::npos) return -1;
    else{
        int number_idx=prefix_idx+prefix.size();
        int size=(int)namelower.size()-number_idx;
        std::string numberStr=namelower.substr(number_idx,size);
        return atoi(numberStr.c_str());
    }
    
    
}
bool ZDXFPluginFactory::isSliceLayer(const std::string& layername)
{
    std::string layernameLower{layername};
    std::transform(layername.begin(), layername.end(), layernameLower.begin(), ::tolower);
    if (layernameLower.find("slice") != std::string::npos)
    {
        DLOG(INFO) << "ZDXFPluginFactory [test]---->get sliceLayer:" << layername;
        return true;
    }
    else
    {
        return false;
    }
}
bool ZDXFPluginFactory::isShuffleChannel(const std::string& layername)
{
    std::string layernameLower{layername};
    std::transform(layername.begin(), layername.end(), layernameLower.begin(), ::tolower);
    if (layernameLower.find("shuffle") != std::string::npos)
    {
        DLOG(INFO) << "ZDXFPluginFactory [test]---->get shuffleChannellayer:" << layername;
        return true;
    }
    else
    {
        return false;
    }
}
nvinfer1::IPluginV2IOExt* ZDXFPluginFactory::createPlugin(const char* name, const nvinfer1::Weights* weights,
        int nbWeights,
        const char* libNamespace)
{
    std::string layername{name};
    if (isShuffleChannel(layername))
    {
        static const int default_group = 2;
        return new ShuffleChannelPlugin(name, default_group);
    }
    else if(isSliceLayer(layername)){
            int layerIdx=getLayerIndex(layername,"slice");
            CHECK_GE(layerIdx,0);
            if(layerIdx<6){
                return new SliceLayerPlugin(name,std::vector<int>{24},0);
            }else if(layerIdx>=6&&layerIdx<14){
                return new SliceLayerPlugin(name,std::vector<int>{48},0);
            }else if(layerIdx>=14 &&layerIdx<17){
                return new SliceLayerPlugin(name,std::vector<int>{96},0);
            }else{
                LOG(FATAL)<<"cannot get  carrect layer index from layer name:" <<layername;
            }

    }
    else
    {
        return nullptr;
    }
}
};