/*
 * @FilePath: /modelTRT_custom_plugin/plugins/src/shuffleChannel.cpp
 * @Brief:
 * @Version: 1.0
 * @Date: 2021-03-02 03:21:52
 * @Author: WangJiaHao
 * @Copyright: IChinae(BEIJING) TECH.CO.,LTD.
 * @LastEditors: Wangjiahao
 * @LastEditTime: 2021-03-15 03:12:09
 */
#include<NvInfer.h>
#include <string.h>
#include<glog/logging.h>
#include <algorithm>
#include"shuffleChannel.hpp"
#include"shuffleChannelkernel.h"
#include"bufferOps.hpp"
#include<cuda_runtime.h>
#include<glog/logging.h>
#include"common.hpp"
#include<cassert>
namespace nvinfer1
{
namespace
{
static const char* SHUFFLE_PLUGIN_VERSION{"1"};
static const char* SHUFFLE_PLUGIN_NAME{"CustomShuffleChannelPlugin"};
}

PluginFieldCollection ShuffleChannelPluginCreator::fc_{};
std::vector<PluginField> ShuffleChannelPluginCreator::pluginAttributes_;

ShuffleChannelPlugin::ShuffleChannelPlugin(const std::string& name, int group): layerName_(name), group_(group) {};
ShuffleChannelPlugin::ShuffleChannelPlugin(const std::string& name, const void* data, size_t length)
{
    DLOG(INFO)<<"-------------create------------";
    const char* d = static_cast<const char*>(data);
    const char* a = d;
    group_ = readFromBuffer<int>(d);
    shape_ = readFromBuffer<nvinfer1::Dims>(d);
    DLOG(INFO)<<"layer: name:"<<name<<"group:"<<group_<<"shape:"<<print_dim(shape_);
    dtype_=(nvinfer1::DataType)readFromBuffer<int>(d);
    CHECK_EQ(d,a + length);
}
const char* ShuffleChannelPlugin::getPluginType() const
{
    DLOG(INFO)<<"-------------getPluginType------------";
    return SHUFFLE_PLUGIN_NAME;
}
const char* ShuffleChannelPlugin::getPluginVersion() const
{
    DLOG(INFO)<<"-------------getPluginVersion------------";
    return SHUFFLE_PLUGIN_VERSION;
}
int ShuffleChannelPlugin::getNbOutputs() const
{
    DLOG(INFO)<<"-------------getNbOutputs------------";

    return 1;
}
//shufflechanneldo not change the dimensions of input
Dims ShuffleChannelPlugin::getOutputDimensions(int index, const Dims* inputs, int nbInputDims)
{
    DLOG(INFO)<<"-------------getOutputDimensions------------";

    CHECK_EQ(nbInputDims ,1);
    CHECK_EQ(index ,0);
    return *inputs;
}
int ShuffleChannelPlugin::initialize()
{
    DLOG(INFO)<<"-------------initialize------------";

    return 0;
}
int ShuffleChannelPlugin::enqueue(int batchSize, const void* const* inputs, void** outputs, void*, cudaStream_t stream)
{
    DLOG(INFO)<<"-------------enqueue------------";
    DLOG(INFO)<<"layer:"<<layerName_<<" batchSize:"<<batchSize<<" group:"<<group_<<" datatype:"<<(int)dtype_;
    DLOG(INFO)<<"layer:"<<layerName_<<" dims:"<<print_dim(shape_);

    int status = 0;
    for(int i=0;i!=batchSize;i++){
        void* output = outputs[i];
        status += shuffleChannelForward(stream, group_, shape_, dtype_, inputs[i], output);
    }
    return status;
}
size_t ShuffleChannelPlugin::getSerializationSize() const
{
    DLOG(INFO)<<"-------------getSerializationSize------------";
    return sizeof(int)+sizeof(nvinfer1::Dims)+sizeof(int);
}
void ShuffleChannelPlugin::serialize(void* buffer)const
{
    DLOG(INFO)<<"-------------serialize------------";
    DLOG(INFO)<<"serialize layer:"<<layerName_<<" group:"<<group_<<" datatype:"<<(int)dtype_;
    DLOG(INFO)<<"serialize layer:"<<layerName_<<" dims:"<<print_dim(shape_);
    char* d = static_cast<char*>(buffer);
    const char* a = d;
    writeToBuffer(d, group_);
    writeToBuffer(d,shape_);
    writeToBuffer(d,(int)dtype_);
    CHECK_EQ(d ,a + getSerializationSize() );
}
//only support float and nchw format for now!!!!
void ShuffleChannelPlugin::configurePlugin(const PluginTensorDesc* in, int nbInput, const PluginTensorDesc* out, int nbOutput)
{
    DLOG(INFO)<<"-------------configurePlugin------------";
    DLOG(INFO)<<"input dim:"<<print_dim(in[0].dims);
    CHECK_EQ(nbOutput, 1);
    CHECK_EQ(nbInput,1);
    assert(in[0].dims.nbDims >= 3);
    auto type=in[0].type;
    assert(type == DataType::kFLOAT || type == DataType::kINT8);
    assert(type==out[0].type);
    assert(in[0].format==out[0].format);
    assert(in[0].format==PluginFormat::kLINEAR);
    memcpy(&shape_, &(in[0].dims), sizeof(Dims));
    dtype_ = type;
}

//supportsFormatCombination(int pos, const PluginTensorDesc* inOut, int nbInputs, int nbOutputs) 
bool ShuffleChannelPlugin::supportsFormatCombination(int pos, const PluginTensorDesc* inOut, int nbInputs, int nbOutputs) const
{
    DLOG(INFO)<<"-------------supportsFormat------------";
    CHECK_EQ(nbInputs, nbOutputs);
    CHECK_EQ(nbInputs,1);
    CHECK_LE(pos,nbInputs+nbOutputs);
    bool condition=true;
    if (inOut[pos].format != PluginFormat::kNCHW)
       condition&=false;
    else
    {
        auto type=inOut[pos].type;
        if (type == DataType::kFLOAT || type == DataType::kINT8)
              condition&=true;
        else
            condition&=false;
    }
    return condition;
}
 nvinfer1::DataType ShuffleChannelPlugin::getOutputDataType(int index, const nvinfer1::DataType* input_types, int num_inputs)const {
     return input_types[0];
 }
void ShuffleChannelPlugin::terminate() {}
void ShuffleChannelPlugin::destroy()
{
    // This gets called when the network containing plugin is destroyed
    delete this;
}
IPluginV2IOExt* ShuffleChannelPlugin::clone() const
{
    DLOG(INFO)<<"-------------clone------------";
    auto plugin = new ShuffleChannelPlugin(this->layerName_, this->group_);
    plugin->setPluginNamespace(namespace_.c_str());
    plugin->dtype_=this->dtype_;
    plugin->shape_=this->shape_;
    return plugin;
}
void ShuffleChannelPlugin::setPluginNamespace(const char* libNamespace)
{
    DLOG(INFO)<<"-------------setPluginNamespace------------";
    DLOG(INFO)<<"libnamespace:"<<libNamespace;
    namespace_ = libNamespace;
}

const char* ShuffleChannelPlugin::getPluginNamespace() const
{
    DLOG(INFO)<<"-------------getPluginNamespace------------";
    return namespace_.c_str();
}
ShuffleChannelPluginCreator:: ShuffleChannelPluginCreator()
{
    pluginAttributes_.emplace_back(PluginField("group", nullptr, PluginFieldType::kINT32, 1));
    fc_.nbFields = pluginAttributes_.size();
    fc_.fields = pluginAttributes_.data();
}

const char* ShuffleChannelPluginCreator::getPluginName() const
{
    return SHUFFLE_PLUGIN_NAME;
}

const char* ShuffleChannelPluginCreator::getPluginVersion() const
{
    return SHUFFLE_PLUGIN_VERSION;
}

const PluginFieldCollection* ShuffleChannelPluginCreator::getFieldNames()
{
    return &fc_;
}

IPluginV2IOExt* ShuffleChannelPluginCreator::createPlugin(const char* name, const PluginFieldCollection* fc)
{
    int group = 2;
    const PluginField* fields = fc->fields;

    // Parse fields from PluginFieldCollection

    assert(fc->nbFields == 1);
    for (int i = 0; i < fc->nbFields; i++)
    {
        if (strcmp(fields[i].name, "group") == 0)
        {
            assert(fields[i].type == PluginFieldType::kINT32);
            group = *(static_cast<const int*>(fields[i].data));
        }
    }
    return new ShuffleChannelPlugin(name, group);
}

IPluginV2IOExt* ShuffleChannelPluginCreator::deserializePlugin(const char* name, const void* serialData, size_t serialLength)
{
    // This object will be deleted when the network is destroyed, which will
    // call ShuffleChannelPlugin::destroy()
    return new ShuffleChannelPlugin(name, serialData, serialLength);
}

void ShuffleChannelPluginCreator::setPluginNamespace(const char* libNamespace)
{
    namespace_ = libNamespace;
}

const char* ShuffleChannelPluginCreator::getPluginNamespace() const
{
    return namespace_.c_str();
}

};