#include"shuffleChannelkernel.h"
#include"cudahelperOps.hpp"
#include"common.hpp"
#include<glog/logging.h>
template<typename DType> 
__global__ void shuffleChannelKernel(const int nthreads, const int feature_map_size,
	const DType *input,  DType *output, int group_row, int group_column, int len) {
	CUDA_KERNEL_LOOP(index, nthreads) {
		const int n = index / group_row / group_column / len;
		const int i = (index / group_column / len) % group_row;
		const int j = index / len % group_column;
		const int k = index - (n * feature_map_size + (i * group_column + j) * len);
		DType* p_o = output + n * feature_map_size + (j * group_row + i) * len;
		p_o[k] = input[index];
	}
}

int shuffleChannelForward(cudaStream_t stream, int channel_groups, nvinfer1::Dims& dimention,nvinfer1::DataType dtype,const void* input,
    void* output){
            int group_row=channel_groups;
            int num=0;
            int feature_size=0;
            int sp_sz=0;
            int channels=0;
            if(dimention.nbDims==3){
                num=1;
                feature_size=dimention.d[0]*dimention.d[1]*dimention.d[2];
                channels=dimention.d[0];
                sp_sz=dimention.d[1]*dimention.d[2];
            }else if(dimention.nbDims==4){
                num=dimention.d[0];
                feature_size=dimention.d[1]*dimention.d[2]*dimention.d[3];
                channels=dimention.d[1];
                sp_sz=dimention.d[2]*dimention.d[3];
            }else{
                LOG(FATAL)<<"[shuffleChannelForward] dims.nbDims==3 || dims.nbDims==4 current dims:"<<print_dim(dimention);
            }
            int group_cols=int(channels/group_row);
            CHECK_EQ(group_cols*group_row,channels)<<"[shuffleChannelForward] group:"<<group_row<<" channels:"<<channels<<" wrong!";
            int volume=channels*sp_sz*num;
            switch(dtype){
                
                case nvinfer1::DataType::kFLOAT:{
                    shuffleChannelKernel<float><<<CUDA_GET_BLOCKS(volume), CUDA_THREADS,0,stream>>>(volume,feature_size,
                                                                                                                                                            static_cast<const float*> (input),  
                                                                                                                                                            static_cast<float*> (output),
                                                                                                                                                            group_row,group_cols,sp_sz);
                    break;
                }
                case nvinfer1::DataType::kINT8:{
                    shuffleChannelKernel< char><<<CUDA_GET_BLOCKS(volume), CUDA_THREADS,0,stream>>>(volume,feature_size,
                                                                                                                                                            static_cast<const char*> (input),  
                                                                                                                                                            static_cast<char*> (output),
                                                                                                                                                            group_row,group_cols,sp_sz);
                    break;
                }case nvinfer1::DataType::kINT32:{
                    shuffleChannelKernel< int><<<CUDA_GET_BLOCKS(volume), CUDA_THREADS,0,stream>>>(volume,feature_size,
                                                                                                                                                            static_cast<const int*> (input),  
                                                                                                                                                            static_cast<int*> (output),
                                                                                                                                                            group_row,group_cols,sp_sz);
                    break;

                }default:{
                    LOG(FATAL)<<"unknown datatype: "<<(int)dtype<<" see nvinfer1::DataType for more info";
                    break;
                }
            }
            return 0;
    }

