/*
 * @FilePath: /modelTRT_custom_plugin/plugins/src/sliceLayer.cpp
 * @Brief:
 * @Version: 1.0
 * @Date: 2021-03-09 06:35:43
 * @Author: WangJiaHao
 * @Copyright: IChinae(BEIJING) TECH.CO.,LTD.
 * @LastEditors: Wangjiahao
 * @LastEditTime: 2021-03-15 06:00:38
 */
#include"sliceLayer.hpp"
#include"bufferOps.hpp"
#include"sliceLayerKernel.h"
#include<glog/logging.h>
#include<cassert>
#include<cstring>
#define MAX_NUM_SLICE_POINTS 16
namespace nvinfer1
{
namespace
{
static const char* SLICELAYER_PLUGIN_VERSION{"1"};
static const char* SLICELAYER_PLUGIN_NAME{"CustomSliceLayerPlugin"};
};

SliceLayerPlugin::SliceLayerPlugin(const std::string& name, const std::vector<int>& slice_points, const int axis):slice_points_(slice_points),name_(name),axsis_(axis)
{
        LOG(INFO)<<name<<" start";
        inputDims_.nbDims=-1;
}
SliceLayerPlugin::SliceLayerPlugin(const std::string& name, const void* data, size_t length)
{
//storage: axsis_|num of slice points|slicePoints_
        LOG(INFO)<<name<<" create";
        const char * dataBytes=static_cast<const char *>(data);
        const char *head=dataBytes;
        axsis_=readFromBuffer<int>(dataBytes);
        int numsOfSlicePoints=readFromBuffer<int>(dataBytes);
        DLOG(INFO)<<"numsOFSlicePoints:"<<numsOfSlicePoints;
        assert(numsOfSlicePoints<=MAX_NUM_SLICE_POINTS);
        for(int i=0;i!=numsOfSlicePoints;i++){
                slice_points_.push_back(readFromBuffer<int>(dataBytes));
        }
        DLOG(INFO)<<"Points:";
        for(auto&i : slice_points_){
                DLOG(INFO)<<"point:"<<i;
        }
        inputDims_=readFromBuffer<nvinfer1::Dims>(dataBytes);
        datatype_=(nvinfer1::DataType)readFromBuffer<int>(dataBytes);
        
        initOutputDims(&inputDims_);
        assert(dataBytes==head+length);

        
}
int SliceLayerPlugin:: getNbOutputs() const
{
        LOG(INFO)<<"getNbOutputs:"<<(int)(slice_points_.size()+1)<<" points:"<<slice_points_[0];
        return (int)(slice_points_.size()+1);
}
bool SliceLayerPlugin::initOutputDims(const Dims *inputs){
        DLOG(INFO)<<"DEBUG2:"<<inputs->nbDims;
        assert(slice_points_.size()!=0);
        DLOG(INFO)<<"DEBUG3";
        assert(inputs!=nullptr);
        DLOG(INFO)<<"DEBUG1";
        if(inputs->nbDims<=0) return false;
        DLOG(INFO)<<"DEBUG12";
        auto inputDim=inputs[0];
        nums_slices_=1;
        slice_size_=1;
        DLOG(INFO)<<"axsis_:"<<axsis_<<"dims:"<<inputDim.nbDims;
        assert(axsis_<inputDim.nbDims);
        for(int i=0;i!=axsis_;i++){
                nums_slices_*=inputDim.d[i];
        }
        for(int i=axsis_+1;i!=inputDim.nbDims;i++){
                slice_size_*=inputDim.d[i];
        }
        std::vector<int> slices;
        for(int i=0;i!=getNbOutputs();i++){
                Dims outputDim;
                memcpy(&outputDim,&inputDim,sizeof(Dims));
                outputDims_.push_back(outputDim);
        }
        const int input_slice_axis =inputDim.d[axsis_];
        int count=0;
        int prev = 0;
        for(size_t i=0;i!=slice_points_.size();i++){
                assert(slice_points_[i]>prev);
                slices.push_back(slice_points_[i]-prev);
                prev=slice_points_[i];

        }
        slices.push_back(input_slice_axis-prev);
        for(size_t i=0;i!=outputDims_.size();i++){
                outputDims_[i].d[axsis_] = slices[i];
                count+=DimsCount(outputDims_[i]);
        }
        assert(count==DimsCount(inputDim));  
        DLOG(INFO)<<"over";    
        return true;
}
Dims SliceLayerPlugin::getOutputDimensions(int index, const Dims* inputs, int nbInputDims)
{
        DLOG(INFO)<<"getOutputDimensions: index:"<<index<<print_dim(inputs[0]);
        assert(nbInputDims ==1);
        assert((index<(int)(slice_points_.size()+1))&& (index>=0));
        DLOG(INFO)<<"output:"<<outputDims_.size();
        if(outputDims_.empty()){
                DLOG(INFO)<<"init output:"<<inputs->nbDims;
                initOutputDims(inputs);
                DLOG(INFO)<<"init output over";
        }
        LOG_IF(FATAL,outputDims_.empty())<<"outputDims_ empty";
        DLOG(INFO)<<"output:"<<outputDims_.size()<<" index:"<<index;
        DLOG(INFO)<<"sliceLayer:"<<name_<<"output index:"<<index<<"outputDim:"<<print_dim( outputDims_[index]);
        return outputDims_[index];        
        
}
int SliceLayerPlugin::initialize()
{
return 0;
}
void SliceLayerPlugin::terminate()
{

}

int SliceLayerPlugin::enqueue(int batchSize, const void* const* inputs, void** outputs, void* workspace,
                              cudaStream_t stream)
{
        DLOG(INFO)<<" enqueue:layer:"<<name_<<" batchSize:"
                                <<batchSize<<" axsis_:"<<axsis_<<
                                " nums_slices_:"<<nums_slices_<<"slice_size_:"<<slice_size_<<" datatype_:"<<(int)datatype_;
        DLOG(INFO)<<"slice_points:";
        for(auto & i: slice_points_){
                DLOG(INFO)<<"points:"<<i;
        }                        
        DLOG(INFO)<<" enqueue:layer:"<<name_<<" inputDims:"<<print_dim(inputDims_);
        DLOG(INFO)<<" enqueue:layer:"<<"outPutDim:";
        for(auto& i : outputDims_){
                DLOG(INFO)<<print_dim(i);
        }
        int status=0;
        for(int i=0;i!=batchSize;i++){
                status+=SliceLayerForward(inputs[i],outputs[i],stream,axsis_,slice_size_,nums_slices_,outputDims_,inputDims_,datatype_);
        }
        return status;
}

size_t SliceLayerPlugin::getSerializationSize() const
{
        //sizeof(slice_points)+sizeof(num of silce points)+sizeof(axsis)+sizeof(inputDims)+sizeof(datatype)
        return sizeof(int)*(slice_points_.size()+3)+sizeof(inputDims_);
}

void SliceLayerPlugin::serialize(void* buffer) const
{
        DLOG(INFO)<<"------------serialize-----";
        DLOG(INFO)<<" serialize:layer:"<<name_<<" axsis_:"<<axsis_<<
                                " nums_slices_:"<<nums_slices_<<" slice_size:"<<slice_size_<<" datatype_:"<<(int)datatype_;
        DLOG(INFO)<<"slice_points:";
        for(auto & i: slice_points_){
                DLOG(INFO)<<"points:"<<i;
        }                        
        DLOG(INFO)<<" serialize:layer:"<<name_<<" inputDims:"<<print_dim(inputDims_);
        DLOG(INFO)<<" serialize:layer:"<<"outPutDim:";
         for(auto& i : outputDims_){
                DLOG(INFO)<<print_dim(i);
        }

        char* d = static_cast<char*>(buffer);
        const char* a = d;
        writeToBuffer(d, axsis_);
        int numSlicePoints=(int)(slice_points_.size());
        writeToBuffer(d,numSlicePoints);
        for(const int &point:slice_points_){
                writeToBuffer(d,point);
        }
        writeToBuffer(d,inputDims_);
        writeToBuffer(d,(int)datatype_);
    assert(d == (a + getSerializationSize()) );
}

void SliceLayerPlugin::configurePlugin(const PluginTensorDesc* in, int nbInput, const PluginTensorDesc* out, int nbOutput)
{
    DLOG(INFO)<<"configurePlugin";
    CHECK_EQ(nbInput,1);
    CHECK_EQ(nbOutput,slice_points_.size()+1);
    assert((in[0].type == DataType::kFLOAT )|| (in[0].type == DataType::kINT8));
    assert(in[0].format == PluginFormat::kNCHW);
    assert(in[0].dims.nbDims >= 3);
    memcpy(&inputDims_,&in[0].dims,sizeof(Dims));
    datatype_ = in[0].type ;
    if(outputDims_.empty()){
            initOutputDims(&inputDims_);
    }
}
nvinfer1::DataType SliceLayerPlugin::getOutputDataType(int index, const nvinfer1::DataType* input_types, int num_inputs) const {
        return input_types[0];
}
bool SliceLayerPlugin::supportsFormatCombination(int pos, const PluginTensorDesc* inOut, int nbInputs, int nbOutputs)  const
{
        DLOG(INFO)<<"slice format check";
        CHECK_LT(pos,nbInputs+nbOutputs);
        CHECK_EQ(nbInputs,1);
        CHECK_EQ(nbOutputs,slice_points_.size()+1);
        DLOG(INFO)<<"after check";
        bool condition=true;
    
        auto format=inOut[pos].format;
        auto type=inOut[pos].type;
        if (format != PluginFormat::kNCHW){
                condition&=false;
                LOG(WARNING)<<"layer:"<<name_<<"dataformat:"<<(int)PluginFormat::kNCHW<<" index:"<<pos;
        }
        else
        {
                if ((type == DataType::kFLOAT) ||(type == DataType::kINT8))
                        condition&=true;
                else{
                        condition&=false;
                                LOG(WARNING)<<"layer:"<<name_<<"dataTye:"<<(int)type<<" index:"<<pos;
                        }
        }
        return condition;
}

const char* SliceLayerPlugin::getPluginType() const
{
        return SLICELAYER_PLUGIN_NAME;
}

const char* SliceLayerPlugin::getPluginVersion() const
{
        return SLICELAYER_PLUGIN_VERSION;
}

void SliceLayerPlugin::destroy()
{
 delete this;
}

nvinfer1::IPluginV2IOExt* SliceLayerPlugin::clone() const
{
        DLOG(INFO)<<"-------------clone:"<<name_<<"------------------";
        auto othr=new SliceLayerPlugin(name_,slice_points_,axsis_);
        othr->setPluginNamespace(this->namespace_.c_str());
        
        othr->inputDims_=this->inputDims_;
        othr->datatype_=this->datatype_;
        if((othr->inputDims_.nbDims>0))
                othr->initOutputDims(&othr->inputDims_);
        DLOG(INFO)<<"othr:outPutDims:"<<othr->outputDims_.size();
        for(auto& i : othr->outputDims_){
                DLOG(INFO)<<print_dim(i);
        }

        return othr;
}

void  SliceLayerPlugin::setPluginNamespace(const char* pluginNamespace)
{
        namespace_=pluginNamespace;
}

const char*  SliceLayerPlugin::getPluginNamespace() const
{
        return namespace_.c_str();
}


SliceLayerPluginCreator::SliceLayerPluginCreator(){
    pluginAttributes_.emplace_back(PluginField("axsis", nullptr, PluginFieldType::kINT32, 1));
    pluginAttributes_.emplace_back(PluginField("num_points", nullptr, PluginFieldType::kINT32, 1));
    pluginAttributes_.emplace_back(PluginField("slice_points", nullptr, PluginFieldType::kINT32, MAX_NUM_SLICE_POINTS));
    fc_.nbFields = pluginAttributes_.size();
    fc_.fields = pluginAttributes_.data();
}
 const char* SliceLayerPluginCreator::getPluginName() const{
         return SLICELAYER_PLUGIN_NAME;
 }

const char* SliceLayerPluginCreator::getPluginVersion() const {
        return SLICELAYER_PLUGIN_VERSION;
}

const PluginFieldCollection* SliceLayerPluginCreator::getFieldNames() {
        return &fc_;
}


nvinfer1::PluginFieldCollection SliceLayerPluginCreator::fc_{};
std::vector<PluginField> SliceLayerPluginCreator::pluginAttributes_{};
//call when parse the original model
IPluginV2IOExt* SliceLayerPluginCreator::createPlugin(const char* name, const PluginFieldCollection* fc){
        int axsis=1;
        int numsOfSlicePoints=-1;
        const int * sliceRawData=nullptr;
        std::vector<int>slice_points;
        IPluginV2IOExt * res=nullptr;
        const PluginField* fields = fc->fields;
        assert(fc->nbFields == 3);
        for (int i = 0; i < fc->nbFields; i++)
        {
                if (strcmp(fields[i].name, "axsis") == 0)
                {
                        assert(fields[i].type == PluginFieldType::kINT32);
                        axsis = *(static_cast<const int*>(fields[i].data));
                }else if(strcmp(fields[i].name, "num_points") == 0){
                        assert(fields[i].type == PluginFieldType::kINT32);
                        numsOfSlicePoints=*(static_cast<const int*>(fields[i].data));
                }else if(strcmp(fields[i].name, "slice_points") == 0){
                        sliceRawData=static_cast<const int*>(fields[i].data);
                }
        }
    if(numsOfSlicePoints>0 && sliceRawData!=nullptr){
            for(int i=0;i!=numsOfSlicePoints;i++){
                    slice_points.push_back(sliceRawData[i]);
            }
            res= new SliceLayerPlugin(name,slice_points,axsis);
    } 
    return res;
}

IPluginV2IOExt* SliceLayerPluginCreator::deserializePlugin(const char* name, const void* serialData, size_t serialLength){
            return new SliceLayerPlugin(name,serialData,serialLength);
}

void SliceLayerPluginCreator::setPluginNamespace(const char* pluginNamespace){
        namespace_=pluginNamespace;
}

const char* SliceLayerPluginCreator::getPluginNamespace() const {
        return namespace_.c_str();
}

};
