#include"sliceLayerKernel.h"
#include"common.hpp"
#include"cudahelperOps.hpp"
template <typename dType>
__global__ void Slice(const int nthreads,
                                const dType *in_data,
                                const int num_slices,
                                const int slice_size,
                                const int input_slices_axsis,
                                const int output_slice_axsis,
                                const int offset_slice_axis,
                                dType * out_data) {

                        CUDA_KERNEL_LOOP(index,nthreads){
                                const int total_slice_size=slice_size*output_slice_axsis;
                                const int slice_num=index/total_slice_size;
                                const int slice_index=index%total_slice_size;
                                const int  input_index=slice_index+(slice_num*output_slice_axsis+offset_slice_axis)*slice_size;
                                out_data[index]=in_data[input_index];
                        }

}

int SliceLayerForward(const void *input,
    void *outputs,
    cudaStream_t stream,
    const int slice_axsis,
    const int slice_size,
    const int num_slice,
    const std::vector<nvinfer1::Dims>&outDims,
    const nvinfer1::Dims inputDims,
    nvinfer1::DataType dtype){
        int offset_slice_axis = 0;
        const int input_slice_axsis=inputDims.d[slice_axsis];
        void *outputHead=outputs;
        for(int i=0;i!=outDims.size();i++){
            const int out_slice_axis=outDims[i].d[slice_axsis];
            const int out_slice_size=out_slice_axis*slice_size;
            const int nthreads=out_slice_size*num_slice;
            switch(dtype){
                case nvinfer1::DataType::kFLOAT:{
                    //run kernel
                    Slice<float><<< CUDA_GET_BLOCKS(nthreads),CUDA_THREADS>>>(
                                                        nthreads,
                                                        static_cast<const float*> (input),
                                                        num_slice,
                                                        slice_size,
                                                        input_slice_axsis,
                                                        out_slice_axis,
                                                        offset_slice_axis,
                                                        static_cast<float*> (outputHead)
                                                        );
                    outputHead=reinterpret_cast<void *>(static_cast<char *>(outputHead)+DimsCount(outDims[i])*sizeof(float));
                    break;
                }case nvinfer1::DataType::kINT8:{
                    //run kernel
                    Slice<char><<< CUDA_GET_BLOCKS(nthreads),CUDA_THREADS>>>(nthreads,
                        static_cast<const char*> (input),
                                                    num_slice,
                                                    slice_size,
                                                    input_slice_axsis,
                                                    out_slice_axis,
                                                    offset_slice_axis,
                                                    static_cast<char*>(outputHead)
                                                     );
                        outputHead=reinterpret_cast<void *>(static_cast<char *>(outputHead)+DimsCount(outDims[i])*sizeof(char));
                        break;
                }case nvinfer1::DataType::kINT32:{
                     //run kernel
                     Slice<int><<< CUDA_GET_BLOCKS(nthreads),CUDA_THREADS>>>(nthreads,
                        static_cast<const int*> (input),
                                                    num_slice,
                                                    slice_size,
                                                    input_slice_axsis,
                                                    out_slice_axis,
                                                    offset_slice_axis,
                                                    static_cast<int *>(outputHead)
                                                     );
                        outputHead=reinterpret_cast<void *>(static_cast<int *>(outputHead)+DimsCount(outDims[i])*sizeof(int));
                        break;
                }
                default:{
                    return -1;
                }
            }
            offset_slice_axis+=out_slice_axis;
        }
        return 0;
}